package com.cicdi.notify.template;

import com.cicdi.notify.DefaultNotifyType;
import com.cicdi.notify.NotifyType;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 通知模板管理器抽象实现
 * <p>在SpringBoot框架中，可以实现BeanPostProcessor接口实现服务商的自动注册</p>
 *
 * @author xueye
 * @see Template
 * @see TemplateProvider
 */
public abstract class AbstractTemplateManager implements TemplateManager {

    /**
     * 所有注册支持的服务商
     * <p>使用{@link #register(TemplateProvider)}方法进行注册</p>
     */
    protected Map<String, Map<String, TemplateProvider>> providers = new ConcurrentHashMap<>();

    /**
     * 通知模板
     */
    protected Map<String, Template> templates = new ConcurrentHashMap<>();

    /**
     * 根据ID获取通知模板配置属性
     *
     * @param type 通知类型 {@link DefaultNotifyType}
     * @param id   通知模板属性ID
     * @return 通知模板属性
     * @see DefaultNotifyType
     */
    protected abstract TemplateProperties getProperties(NotifyType type, String id);

    @Override
    public Template getTemplate(NotifyType type, String templateId) {
        Template template;
        if (null != templates.get(templateId)) {
            template = templates.get(templateId);
        } else {
            template = createTemplate(type, getProperties(type, templateId));
            if (null == template) {
                throw new UnsupportedOperationException("不支持的通知类型: " + type.getId());
            }
            templates.put(templateId, template);
        }
        return template;
    }

    @Override
    public Template createTemplate(NotifyType type, TemplateProperties properties) {
        Map<String, TemplateProvider> providerTypes = providers.get(type.getId());
        if (null == providerTypes) {
            throw new UnsupportedOperationException("不支持的通知类型: " + properties.getType());
        }
        TemplateProvider templateProvider = providerTypes.get(properties.getProvider());
        if (null == templateProvider) {
            throw new UnsupportedOperationException("不支持的服务商: " + properties.getProvider());
        }
        return templateProvider.createTemplate(properties);
    }

    @Override
    public void reload(String templateId) {
        templates.remove(templateId);
    }

    @Override
    public void register(TemplateProvider provider) {
        providers.computeIfAbsent(provider.getType().getId(), ignore -> new ConcurrentHashMap<>()).put(provider.getProvider().getId(), provider);
    }

}
